unit rmGauge;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms;

type
  TOrientation = (otHorizontal, otVertical);
  TBorderStyle = (bsNone, bsSingle, bsRaised, bsLowered);
  TGaugeStyle = (gsBar, gsPoint);
  TColorStyle = (csMonoColorVar, csMonoColorFix, csMultiColor);

  TrmGauge = class(TGraphicControl)
  private
    { Dclarations prives }
    FBmpBuffer: TBitmap;
    PeakVal: integer;
    FBGPen: TPen;
    FBGBrush: TBrush;
    FGaugeStyle: TGaugeStyle;
    FColorStyle: TColorStyle;
    FBorderStyle: TBorderStyle;
    FTransparent: boolean;
    FZone1Color: TColor;
    FZone2Color: TColor;
    FZone3Color: TColor;
    FUnusedColorFix: boolean;
    FUnusedColor: TColor;
    FBorderSpacing: word;
    FPeakMemory: boolean;
    FPeakColor: TColor;
    FPointLength: byte;
    FMinValue: integer;
    FMaxValue: integer;
    FZone1Limit: integer;
    FZone2Limit: integer;
    FPosition: integer;
    FOrientation: TOrientation;
    procedure SetBGBrush(Value: TBrush);
    procedure SetBGPen(Value: TPen);
    procedure SetGaugeStyle(val: TGaugeStyle);
    procedure SetColorStyle(val: TColorStyle);
    procedure SetBorderStyle(val: TBorderStyle);
    procedure SetTransparent(val: boolean);
    procedure SetZone1Color(val: TColor);
    procedure SetZone2Color(val: TColor);
    procedure SetZone3Color(val: TColor);
    procedure SetUnusedColorFix(val: boolean);
    procedure SetUnusedColor(val: TColor);
    procedure SetBorderSpacing(val: word);
    procedure SetPeakMemory(val: boolean);
    procedure SetPeakColor(val: TColor);
    procedure SetPointLength(val: byte);
    procedure SetMinValue(val: integer);
    procedure SetMaxValue(val: integer);
    procedure SetZone1Limit(val: integer);
    procedure SetZone2Limit(val: integer);
    procedure SetPosition(val: integer);
    procedure SetOrientation(val: TOrientation);
    procedure StyleChanged(sender: TObject);
  protected
    { Dclarations protges }
    procedure Paint; override;
    procedure UpdateBmpBuffer;
  public
    { Dclarations publiques }
    constructor Create(anOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
  published
    { Dclarations publies }
    property BGBrush: TBrush read FBGBrush write SetBGBrush;
    property BGPen: TPen read FBGPen write SetBGPen;
    property GaugeStyle: TGaugeStyle read FGaugeStyle write SetGaugeStyle;
    property ColorStyle: TColorStyle read FColorStyle write SetColorStyle;
    property BorderStyle: TBorderStyle read FBorderStyle write SetBorderStyle;
    property Transparent: boolean read FTransparent write SetTransparent;
    property Zone1Color: TColor read FZone1Color write SetZone1Color;
    property Zone2Color: TColor read FZone2Color write SetZone2Color;
    property Zone3Color: TColor read FZone3Color write SetZone3Color;
    property UnusedColorFix: boolean read FUnusedColorFix write SetUnusedColorFix;
    property UnusedColor: TColor read FUnusedColor write SetUnusedColor;
    property BorderSpacing: word read FBorderSpacing write SetBorderSpacing;
    property PeakMemory: boolean read FPeakMemory write SetPeakMemory;
    property PeakColor: TColor read FPeakColor write SetPeakColor;
    property PointLength: byte read FPointLength write SetPointLength;
    property MinValue: integer read FMinValue write SetMinValue;
    property MaxValue: integer read FMaxValue write SetMaxValue;
    property Zone1Limit: integer read FZone1Limit write SetZone1Limit;
    property Zone2Limit: integer read FZone2Limit write SetZone2Limit;
    property Position: integer read FPosition write SetPosition;
    property Orientation: TOrientation read FOrientation write SetOrientation;
    procedure ResetPeakMemory;

    { Standard properties }
    property Hint;
    property ShowHint;
    { Standard events }
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property Visible;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('OptoElec', [TrmGauge]);
end;

{ TrmGauge }

constructor TrmGauge.Create(anOwner: TComponent);
begin
  Inherited Create(anOwner);
  FBmpBuffer := TBitmap.Create;
  FBmpBuffer.OnChange := StyleChanged;
  FBGPen := TPen.Create;
  FBGPen.OnChange := StyleChanged;
  FBGBrush := TBrush.Create;
  FBGBrush.OnChange := StyleChanged;
  FBGBrush.Color := clSilver;
  Height := 17;
  width := 161;
  FColorStyle := csMultiColor;
  FBorderStyle := bsLowered;
  FBorderSpacing := 3;
  FUnusedColorFix := false;
  FUnusedColor := clSilver;
  FZone1Color := clLime;
  FZone2Color := clYellow;
  FZone3Color := clRed;
  FZone1Limit := 60;
  FZone2Limit := 80;
  FPeakMemory := false;
  FPeakColor := clRed;
  FPointLength := 4;
  FMinValue := 0;
  FMaxValue := 100;
  FPosition := 0;
end;

destructor TrmGauge.Destroy;
begin
  If FBmpBuffer <> nil then FBmpBuffer.free;
  FBGPen.free;
  FBGBrush.free;
  Inherited Destroy;
end;

function GetShaddowColor(cl: TColor): TColor;
type
  colorConvert = record
    case byte of
      0 : (z: TColor);
      1 : (a, b, c, d: byte);
  end;
begin
  colorConvert(result).a := colorConvert(cl).a div 2;
  colorConvert(result).b := colorConvert(cl).b div 2;
  colorConvert(result).c := colorConvert(cl).c div 2;
  colorConvert(result).d := colorConvert(cl).d div 2;
end;

procedure TrmGauge.StyleChanged(sender: TObject);
begin
  if FBGPen.Width > 1 then
    FBGPen.Width := 1;
  invalidate;
end;

procedure TrmGauge.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  Inherited ;
  If FBmpBuffer <> NIL then begin
    FBmpBuffer.Destroy;
    FBmpBuffer := TBitmap.Create;
    FBmpBuffer.Height := aHeight;
    FBmpBuffer.Width := aWidth;
  end;
end;

procedure TrmGauge.Paint;
begin
  UpdateBmpBuffer;
  Canvas.Draw(0, 0, FBmpBuffer);
end;

procedure TrmGauge.UpdateBmpBuffer;

  procedure DrawBackGround;
  begin

    With FBmpBuffer do begin
      Canvas.pen.Assign(FBGPen);
      canvas.Brush.Assign(FBGBrush);
      if (FBorderStyle <> bsSingle) then
        Canvas.Pen.Color := canvas.Brush.Color;
      Canvas.rectangle(0, 0, Width, Height);

      // 3D effect ?
      if (FBorderStyle = bsRaised) or (FBorderStyle = bsLowered) then begin
        if FBorderStyle = bsRaised then
          Canvas.Pen.Color := clWhite
        else
          Canvas.pen.color := clGray;
        Canvas.moveTo(0, height - 1);
        Canvas.lineTo(0, 0);
        Canvas.lineTo(width - 1, 0);
        if FBorderStyle = bsRaised then
          Canvas.Pen.Color := clGray
        else
          Canvas.pen.color := clWhite;
        Canvas.lineTo(width - 1, height - 1);
        Canvas.lineTo(0, height - 1);
      end;
      Canvas.Brush.Style := bsSolid;
    end;
  end;

  procedure DrawZone(StartVal, EndVal: integer; cl: TColor);
  begin
    With FBmpBuffer do begin
      Canvas.pen.Width := 0;
      Canvas.pen.color := cl;
      Canvas.brush.color := cl;
      if FOrientation = otHorizontal then
        Canvas.rectangle(StartVal, FBorderSpacing, EndVal, height - FBorderSpacing)
      else if FOrientation = otVertical then
        Canvas.rectangle(FBorderSpacing, height - StartVal, width - FBorderSpacing, height - EndVal);
    end;
    //Canvas.Draw(0, 0, FBmpBuffer);
  end;

  procedure DrawPeakMemory(peak: integer);
  begin
    With FBmpBuffer.Canvas do begin
      Pen.Color := FPeakColor;
      pen.width := 1;
      if FOrientation = otHorizontal then begin
        moveTo(peak, FBorderSpacing);
        lineTo(peak, height - FBorderSpacing);
      end else if FOrientation = otVertical then begin
        moveTo(FBorderSpacing, height - peak);
        lineTo(width - FBorderSpacing, height - peak);
      end;
    end;
  end;

var
  StartZone1, EndZone1, StartZone2, EndZone2, StartZone3, EndZone3, peak: integer;
  ratio: integer;
begin
  // background
  DrawBackGround;

  // if not UnusedColorFix then get shaddowColor for each zone
  if (FColorStyle = csMultiColor) and (not UnusedColorFix) then begin
    StartZone1 := FBorderSpacing;
    try
      ratio := trunc((100 / ((FMaxValue - FMinValue) / (FZone1Limit - FMinValue))));
      if FOrientation = otHorizontal then
        EndZone1 := trunc((width - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing
      else if FOrientation = otVertical then
        EndZone1 := trunc((height - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing;
      DrawZone(StartZone1, EndZone1, GetShaddowColor(FZone1Color));

      StartZone2 := EndZone1;
      ratio := trunc((100 / ((FMaxValue - FMinValue) / (FZone2Limit - FMinValue))));
      if FOrientation = otHorizontal then
        EndZone2 := trunc((width - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing
      else if FOrientation = otVertical then
        EndZone2 := trunc((height - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing;
      DrawZone(StartZone2, EndZone2, GetShaddowColor(FZone2Color));

      StartZone3 := EndZone2;
      ratio := trunc((100 / ((FMaxValue - FMinValue) / (FMaxValue - FMinValue))));
      if FOrientation = otHorizontal then
        EndZone3 := trunc((width - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing
      else if FOrientation = otVertical then
        EndZone3 := trunc((height - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing;
      DrawZone(StartZone3, EndZone3, GetShaddowColor(FZone3Color));
    except
    end;

  // if UnusedColorFix then only one color for unusued color
  end else begin
    if FOrientation = otHorizontal then
      DrawZone(FBorderSpacing, width - FBorderSpacing, FUnusedColor)
    else if FOrientation = otVertical then
      DrawZone(FBorderSpacing, height - FBorderSpacing, FUnusedColor);
  end;

  // Fixed color, color don't depend of position
  // only zone1Color for all value from MinValue to MaxValue
  if (FColorStyle = csMonoColorFix) and (FPosition > FMinValue) then begin

    ratio := trunc((100 / ((FMaxValue - FMinValue) / (FPosition - FMinValue))));

    if GaugeStyle = gsBar then begin
      StartZone1 := FBorderSpacing;
      try
        if FOrientation = otHorizontal then
          EndZone1 := trunc((width - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing
        else if FOrientation = otVertical then
          EndZone1 := trunc((height - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing;
      except
      end;

    end else if GaugeStyle = gsPoint then begin
      StartZone1 := FBorderSpacing;
      try
        if FOrientation = otHorizontal then begin
          StartZone1 := trunc((width - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing - trunc (FPointLength / 2);
          EndZone1 := trunc((width - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing + trunc (FPointLength / 2);
        end else if FOrientation = otVertical then begin
          StartZone1 := trunc((height - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing - trunc (FPointLength / 2);
          EndZone1 := trunc((height - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing + trunc (FPointLength / 2);
        end;
      except
      end;

    end;
    DrawZone(StartZone1, EndZone1, FZone1Color);

  // only one color at a time for all value from MinValue to MaxValue,
  // but color depend of position
  end else if (FColorStyle = csMonoColorVar) and (FPosition > FMinValue) then begin
    StartZone1 := FBorderSpacing;
    try
      ratio := trunc((100 / ((FMaxValue - FMinValue) / (FPosition - FMinValue))));
      // ratio := trunc((100 / (FMaxValue - FMinValue)) * FPosition);
      if FOrientation = otHorizontal then
        EndZone1 := trunc((width - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing
      else if FOrientation = otVertical then
        EndZone1 := trunc((height - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing;
      if FPosition <= FZone1Limit then
        DrawZone(StartZone1, EndZone1, FZone1Color)
      else if (FPosition > FZone1Limit) and (FPosition <= FZone2Limit) then
        DrawZone(StartZone1, EndZone1, FZone2Color)
      else if (FPosition > FZone2Limit) then
        DrawZone(StartZone1, EndZone1, FZone3Color);
    except
    end;

  // one or two or three color at a time, depend of position
  end else if (FColorStyle = csMultiColor) and (FPosition > FMinValue) then begin
    StartZone1 := FBorderSpacing;
    if FPosition <= FZone1Limit then begin
      try
        ratio := trunc((100 / ((FMaxValue - FMinValue) / (FPosition - FMinValue))));
        // ratio := trunc((100 / (FMaxValue - FMinValue)) * FPosition);
        if FOrientation = otHorizontal then
          EndZone1 := trunc((width - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing
        else if FOrientation = otVertical then
          EndZone1 := trunc((height - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing;
        DrawZone(StartZone1, EndZone1, FZone1Color);
      except
      end;
    end else if (FPosition > FZone1Limit) and (FPosition <= FZone2Limit) then begin
      try
        ratio := trunc((100 / ((FMaxValue - FMinValue) / (FZone1Limit - FMinValue))));
        // ratio := trunc((100 / (FMaxValue - FMinValue)) * FZone1Limit);
        if FOrientation = otHorizontal then
          EndZone1 := trunc((width - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing
        else if FOrientation = otVertical then
          EndZone1 := trunc((height - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing;
        DrawZone(StartZone1, EndZone1, FZone1Color);
        StartZone2 := EndZone1;
        ratio := trunc((100 / ((FMaxValue - FMinValue) / (FPosition - FMinValue))));
        // ratio := trunc((100 / (FMaxValue - FMinValue)) * FPosition);
        if FOrientation = otHorizontal then
          EndZone2 := trunc((width - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing
        else if FOrientation = otVertical then
          EndZone2 := trunc((height - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing;
        DrawZone(StartZone2, EndZone2, FZone2Color);
      except
      end;
    end else if (FPosition > FZone2Limit) then begin
      try
        ratio := trunc((100 / ((FMaxValue - FMinValue) / (FZone1Limit - FMinValue))));
        // ratio := trunc((100 / (FMaxValue - FMinValue)) * FZone1Limit);
        if FOrientation = otHorizontal then
          EndZone1 := trunc((width - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing
        else if FOrientation = otVertical then
          EndZone1 := trunc((height - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing;
        DrawZone(StartZone1, EndZone1, FZone1Color);
        StartZone2 := EndZone1;
        ratio := trunc((100 / ((FMaxValue - FMinValue) / (FZone2Limit - FMinValue))));
        // ratio := trunc((100 / (FMaxValue - FMinValue)) * FZone2Limit);
        if FOrientation = otHorizontal then
          EndZone2 := trunc((width - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing
        else if FOrientation = otVertical then
          EndZone2 := trunc((height - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing;
        DrawZone(StartZone2, EndZone2, FZone2Color);
        StartZone3 := EndZone2;
        ratio := trunc((100 / ((FMaxValue - FMinValue) / (FPosition - FMinValue))));
        // ratio := trunc((100 / (FMaxValue - FMinValue)) * FPosition);
        if FOrientation = otHorizontal then
          EndZone3 := trunc((width - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing
        else if FOrientation = otVertical then
          EndZone3 := trunc((height - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing;
        DrawZone(StartZone3, EndZone3, FZone3Color);
      except
      end;
    end;
  end;

  // draw PeakMemory
  if FPeakMemory and (PeakVal > 0)then begin
    try
      ratio := trunc((100 / ((FMaxValue - FMinValue) / (PeakVal - FMinValue))));
      if FOrientation = otHorizontal then
        peak := trunc((width - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing
      else if FOrientation = otVertical then
        peak := trunc((height - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing;
      DrawPeakMemory(peak);
    except
    end;
  end;

end;

procedure TrmGauge.SetBGBrush(Value: TBrush);
begin
  FBGBrush.Assign(Value);
end;

procedure TrmGauge.SetBGPen(Value: TPen);
begin
  FBGPen.Assign(Value);
end;

procedure TrmGauge.SetGaugeStyle(val: TGaugeStyle);
begin
  if val <> FGaugeStyle then begin
    FGaugeStyle := val;
    paint;
  end;
end;

procedure TrmGauge.SetColorStyle(val: TColorStyle);
begin
  if val <> FColorStyle then begin
    FColorStyle := val;
    paint;
  end;
end;

procedure TrmGauge.SetBorderStyle(val: TBorderStyle);
begin
  if val <> FBorderStyle then begin
    FBorderStyle := val;
    paint;
  end;
end;

procedure TrmGauge.SetBorderSpacing(val: word);
begin
  if val <> FBorderSpacing then begin
    if val > ((height div 2) - 1) then
      val := (height div 2) - 1;
    FBorderSpacing := val;
    paint;
  end;
end;

procedure TrmGauge.SetPeakColor(val: TColor);
begin
  if val <> FPeakColor then begin
    FPeakColor := val;
    paint;
  end;
end;

procedure TrmGauge.SetPeakMemory(val: boolean);
begin
  if val <> FPeakMemory then begin
    FPeakMemory := val;
    paint;
  end;
end;

procedure TrmGauge.SetPointLength(val: byte);
begin
  if val <> FPointLength then begin
    FPointLength := val;
    paint;
  end;
end;

procedure TrmGauge.SetMaxValue(val: integer);
begin
  if val <> FMaxValue then begin
    if val < FMinValue then
      FMaxValue := FMinValue
    else
      FMaxValue := val;
    paint;
  end;
end;

procedure TrmGauge.SetMinValue(val: integer);
begin
  if val <> FMinValue then begin
    if val > FMaxValue then
      FMinValue := FMaxValue
    else
      FMinValue := val;
    FMinValue := val;
    if FPosition < FMinValue then
      FPosition := FMinValue;
    paint;
  end;
end;

procedure TrmGauge.SetOrientation(val: TOrientation);
var
  tmp: integer;
begin
  if val <> FOrientation then begin
    FOrientation := val;
    if (csDesigning in ComponentState) and not(csReading in ComponentState) then begin
      tmp := width;
      width := height;
      height := tmp;
    end;
    paint;
  end;
end;

procedure TrmGauge.SetPosition(val: integer);
begin
  if val <> FPosition then begin
    if val < FMinValue then
      val := FMinValue;
    FPosition := val;
    if FPeakMemory and (FPosition > PeakVal) then
      PeakVal := FPosition;
    paint;
  end;
end;

procedure TrmGauge.SetUnusedColor(val: TColor);
begin
  if val <> FUnusedColor then begin
    FUnusedColor := val;
    paint;
  end;
end;

procedure TrmGauge.SetUnusedColorFix(val: boolean);
begin
  if val <> FUnusedColorFix then begin
    FUnusedColorFix := val;
    paint;
  end;
end;

procedure TrmGauge.SetTransparent(val: boolean);
begin
  if val <> FTransparent then begin
    FTransparent := val;
    //paint;
    invalidate;
  end;
end;

procedure TrmGauge.SetZone1Color(val: TColor);
begin
  if val <> FZone1Color then begin
    FZone1Color := val;
    paint;
  end;
end;

procedure TrmGauge.SetZone1Limit(val: integer);
begin
  if val <> FZone1Limit then begin
    FZone1Limit := val;
    paint;
  end;
end;

procedure TrmGauge.SetZone2Color(val: TColor);
begin
  if val <> FZone2Color then begin
    FZone2Color := val;
    paint;
  end;
end;

procedure TrmGauge.SetZone2Limit(val: integer);
begin
  if val <> FZone2Limit then begin
    FZone2Limit := val;
    paint;
  end;
end;

procedure TrmGauge.SetZone3Color(val: TColor);
begin
  if val <> FZone3Color then begin
    FZone3Color := val;
    paint;
  end;
end;

procedure TrmGauge.ResetPeakMemory;
begin
  PeakVal := FPosition;
  paint;
end;

end.
