unit rmScale;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TOrientation = (stLeft, stRight, stTop, stBottom);
  TTextStyle = (tsMinMaxValues, tsMinMaxText);
  TrmScale = class(TGraphicControl)
  private
    { Dclarations prives }
    FBmpBuffer: TBitmap;
    FOrientation: TOrientation;
    FBGColor: TColor;
    FLineColor: TColor;
    FLineActiveColor: TColor;
    FLineInactiveColor: TColor;
    FLineWidth: word;
    FTicksColor: TColor;
    FTicksActiveColor: TColor;
    FTicksInactiveColor: TColor;
    FTicksLength: word;
    FTicksWidth: word;
    FTicksCount: word;
    FTicksOffset: word;
    FTextUnit: string;
    FDecimals: word;
    FShowText: boolean;
    FTextStyle: TTextStyle;
    FTextMin, FTextMax: string;
    FMinValue: single;
    FMaxValue: single;
    FShowTickMarker: boolean;
    FMarkerColor: TColor;
    FPosition: single;
    FBorderSpacing: word;
    FTextLeftOffset: integer;
    FTextRightOffset: integer;
    FTextBottomOffset: integer;
    FTextTopOffset: integer;
    FTextTickSpacing: word;
    FFontName: string;
    FFontColor: TColor;
    FFontSize: word;
    procedure SetOrientation(val: TOrientation);
    procedure SetBGColor(val: TColor);
    procedure SetLineColor(val: TColor);
    procedure SetLineActiveColor(val: TColor);
    procedure SetLineInactiveColor(val: TColor);
    procedure SetLineWidth(val: word);
    procedure SetTicksColor(val: TColor);
    procedure SetTicksActiveColor(val: TColor);
    procedure SetTicksInactiveColor(val: TColor);
    procedure SetTicksLength(val: word);
    procedure SetTicksWidth(val: word);
    procedure SetTicksCount(val: word);
    procedure SetTicksOffset(val: word);
    procedure SetTextUnit(val: String);
    procedure SetDecimals(val: word);
    procedure SetShowText(val: boolean);
    procedure SetTextStyle(val: TTextStyle);
    procedure SetTextMin(val: string);
    procedure SetTextMax(val: string);
    procedure SetMinValue(val: single);
    procedure SetMaxValue(val: single);
    procedure SetPosition(val: single);
    procedure SetShowTickMarker(val: boolean);
    procedure SetMarkerColor(val: TColor);
    procedure SetBorderSpacing(val: word);
    procedure SetTextLeftOffset(val: integer);
    procedure SetTextRightOffset(val: integer);
    procedure SetTextBottomOffset(val: integer);
    procedure SetTextTopOffset(val: integer);
    procedure SetTextTickSpacing(val: word);
    procedure SetFontName(val: string);
    procedure SetFontColor(val: TColor);
    procedure SetFontSize(val: word);
  protected
    { Dclarations protges }
    procedure Paint; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure UpdateBmpBuffer;
  public
    { Dclarations publiques }
    constructor Create(anOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Dclarations publies }
    property Orientation: TOrientation read FOrientation write SetOrientation;
    property BGColor: TColor read FBGColor write SetBGColor;
    property LineColor: TColor read FLineColor write SetLineColor;
    property LineActiveColor: TColor read FLineActiveColor write SetLineActiveColor;
    property LineInactiveColor: TColor read FLineInactiveColor write SetLineInactiveColor;
    property LineWidth: word read FLineWidth write SetLineWidth;
    property TicksColor: TColor read FTicksColor write SetTicksColor;
    property TicksActiveColor: TColor read FTicksActiveColor write SetTicksActiveColor;
    property TicksInactiveColor: TColor read FTicksInactiveColor write SetTicksInactiveColor;
    property TicksLength: word read FTicksLength write SetTicksLength;
    property TicksWidth: word read FTicksWidth write SetTicksWidth;
    property TicksCount: word read FTicksCount write SetTicksCount;
    property TicksOffset: word read FTicksOffset write SetTicksOffset;
    property TextUnit: String read FTextUnit write SetTextUnit;
    property Decimals: word read FDecimals write SetDecimals;
    property ShowText: boolean read FShowText write SetShowText;
    property TextStyle: TTextStyle read FTextStyle write SetTextStyle;
    property TextMin: string read FTextMin write SetTextMin;
    property TextMax: string read FTextMax write SetTextMax;
    property MinValue: single read FMinValue write SetMinValue;
    property MaxValue: single read FMaxValue write SetMaxValue;
    property Position: single read FPosition write SetPosition;
    property ShowTickMarker: boolean read FShowTickMarker write SetShowTickMarker;
    property MarkerColor: TColor read FMarkerColor write SetMarkerColor;
    property BorderSpacing: word read FBorderSpacing write SetBorderSpacing;
    property TextLeftOffset: integer read FTextLeftOffset write SetTextLeftOffset;
    property TextRightOffset: integer read FTextRightOffset write SetTextRightOffset;
    property TextBottomOffset: integer read FTextBottomOffset write SetTextBottomOffset;
    property TextTopOffset: integer read FTextTopOffset write SetTextTopOffset;
    property TextTickSpacing: word read FTextTickSpacing write SetTextTickSpacing;
    property FontName: string read FFontName write SetFontName;
    property FontColor: TColor read FFontColor write SetFontColor;
    property FontSize: word read FFontSize write SetFontSize;
    { Standard properties }
    property Align;
    property Hint;
    property ShowHint;
    property PopupMenu;
    { Standard events }
    property OnClick;
    property OnDblClick;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('OptoElec', [TrmScale]);
end;

{ TrmScale }

function r2s(dec: integer; r: real): string;
var
  s: string;
begin
  str(r:0:dec, s);
  result := s;
end;

constructor TrmScale.Create(anOwner: TComponent);
begin
  Inherited Create(anOwner);
  Height := 193;
  width := 49;
  FTextUnit := '';
  FBmpBuffer := TBitmap.Create;
  FBGColor := clSilver;
  FLineColor := clBlack;
  FLineActiveColor := $004080FF;
  FLineInactiveColor := clMaroon;
  FLineWidth := 1;
  FTicksColor := clBlack;
  FTicksActiveColor := clLime;
  FTicksInactiveColor := clGreen;
  FTicksCount := 11;
  FTicksLength := 8;
  FTicksWidth := 1;
  FTicksOffset := 0;
  FBorderSpacing := 10;
  FTextLeftOffset := 4;
  FTextRightOffset := 4;
  FTextBottomOffset := 4;
  FTextTopOffset := 4;
  FTextTickSpacing := 4;
  FShowText := true;
  FTextStyle := tsMinMaxValues;
  FTextMin := 'Empty';
  FTextMax := 'Full';
  FMinValue := 0;
  FMaxValue := 100;
  FShowTickMarker := true;
  FMarkerColor := clRed;
  FPosition := 30;
  FDecimals := 1;
  FontName := 'arial';
  FontSize := 8;
  FontColor := clBlue;
  FOrientation := stLeft;
end;

destructor TrmScale.Destroy;
begin
  If FBmpBuffer <> NIL then FBmpBuffer.free;
  Inherited Destroy;
end;

procedure TrmScale.Paint;
begin
  UpdateBmpBuffer;
  Canvas.Draw(0, 0, FBmpBuffer);
end;

procedure TrmScale.SetBGColor(val: TColor);
begin
  if val <> FBGColor then begin
    FBGColor := val;
    paint;
  end;
end;

procedure TrmScale.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  Inherited ;
  If FBmpBuffer <> NIL then begin
    FBmpBuffer.Destroy;
    FBmpBuffer := TBitmap.Create;
    FBmpBuffer.Height := aHeight;
    FBmpBuffer.Width := aWidth;
  end;
end;

procedure TrmScale.SetLineColor(val: TColor);
begin
  if val <> FLineColor then begin
    FLineColor := val;
    paint;
  end;
end;

procedure TrmScale.SetLineActiveColor(val: TColor);
begin
  if val <> FLineActiveColor then begin
    FLineActiveColor := val;
    paint;
  end;
end;

procedure TrmScale.SetLineInactiveColor(val: TColor);
begin
  if val <> FLineInactiveColor then begin
    FLineInactiveColor := val;
    paint;
  end;
end;

procedure TrmScale.SetLineWidth(val: word);
begin
  if val <> FLineWidth then begin
    FLineWidth := val;
    paint;
  end;
end;

procedure TrmScale.SetOrientation(val: TOrientation);
var
  tmp: integer;
  oldVal: TOrientation;
begin
  if val <> FOrientation then begin
    oldVal := FOrientation;
    FOrientation := val;
    if ((oldVal = stLeft) and ((val = stTop) or (val = stBottom))) or
       ((oldVal = stRight) and ((val = stTop) or (val = stBottom))) or
       ((oldVal = stTop) and ((val = stLeft) or (val = stRight))) or
       ((oldVal = stBottom) and ((val = stLeft) or (val = stRight)))
    then begin
      if (csDesigning in ComponentState) and not(csReading in ComponentState) then begin
        tmp := width;
        width := height;
        height := tmp;
      end;
    end;
    paint;
  end;
end;

procedure TrmScale.SetShowText(val: boolean);
begin
  if val <> FShowText then begin
    FShowText := val;
    paint;
  end;
end;

procedure TrmScale.SetTextStyle(val: TTextStyle);
begin
  if val <> FTextStyle then begin
    FTextStyle := val;
    paint;
  end;
end;

procedure TrmScale.SetTextMin(val: string);
begin
  if val <> FTextMin then begin
    FTextMin := val;
    paint;
  end;
end;

procedure TrmScale.SetTextMax(val: string);
begin
  if val <> FTextMax then begin
    FTextMax := val;
    paint;
  end;
end;

procedure TrmScale.SetTicksColor(val: TColor);
begin
  if val <> FTicksColor then begin
    FTicksColor := val;
    paint;
  end;
end;

procedure TrmScale.SetTicksActiveColor(val: TColor);
begin
  if val <> FTicksActiveColor then begin
    FTicksActiveColor := val;
    paint;
  end;
end;

procedure TrmScale.SetTicksInactiveColor(val: TColor);
begin
  if val <> FTicksInactiveColor then begin
    FTicksInactiveColor := val;
    paint;
  end;
end;

procedure TrmScale.SetTicksLength(val: word);
begin
  if val <> FTicksLength then begin
    FTicksLength := val;
    paint;
  end;
end;

procedure TrmScale.SetTicksWidth(val: word);
begin
  if val <> FTicksWidth then begin
    FTicksWidth := val;
    paint;
  end;
end;

procedure TrmScale.SetTextUnit(val: String);
begin
  if val <> FTextUnit then begin
    FTextUnit := val;
    paint;
  end;
end;

procedure TrmScale.SetTicksCount(val: word);
begin
  if val <> FTicksCount then begin
    if val < 2 then
      val := 2;
    if val > 255 then
      val := 255;
    FTicksCount := val;
    paint;
  end;
end;

procedure TrmScale.SetTicksOffset(val: word);
begin
  if val <> FTicksOffset then begin
    FTicksOffset := val;
    paint;
  end;
end;

procedure TrmScale.UpdateBmpBuffer;

  procedure DrawBackGround;
  begin
    With FBmpBuffer.Canvas do begin
      Pen.Color := FBGColor;
      Brush.Color := FBGColor;
      rectangle(0, 0, Width, Height);
    end;
  end;

  procedure DrawBaseLine;

    procedure DrawOneLine(x1, y1, x2, y2: integer);
    begin
      With FBmpBuffer.Canvas do begin
        pen.color := FLineColor;
        moveTo(x1, y1);
        lineTo(x2, y2);
      end;
    end;

    procedure DrawOneRect(x1, y1, x2, y2: integer);
    begin
      With FBmpBuffer.Canvas do begin
        pen.color := FLineInactiveColor;
        brush.color := FLineInactiveColor;
        rectangle(x1, y1, x2, y2);
        // 3d effect ?
        if ((x2 - x1) > 2) or ((x1 - x2) > 2) then begin
          pen.color := clGray;
          moveTo(x1, y2);
          lineTo(x1, y1);
          lineTo(x2 - 1, y1);
          pen.color := clWhite;
          lineTo(x2 - 1, y2);
          lineTo(x1, y2);
        end;
      end;
    end;

  var
    X1, X2, Y1, Y2: integer;
  begin
    With FBmpBuffer.Canvas do begin
      pen.color := FLineColor;

      if FOrientation = stLeft then begin
        X1 := 0;
        Y1 := FBorderSpacing;
        X2 := 0;
        Y2 := height - FBorderSpacing + 1;
        if FLineWidth > 1 then
          DrawOneRect(X1, Y1, X2 + FLineWidth, Y2)
        else
          DrawOneLine(X1, Y1, X2, Y2);

      end else if FOrientation = stRight then begin
        X1 := width - 1;
        Y1 := FBorderSpacing;
        X2 := width - 1;
        Y2 := height - FBorderSpacing + 1;
        if FLineWidth > 1 then
          DrawOneRect(X2 - FLineWidth + 1, Y1, X1 + 1, Y2)
        else
          DrawOneLine(X1, Y1, X2, Y2);

      end else if FOrientation = stTop then begin
        X1 := FBorderSpacing;
        Y1 := 0;
        X2 := width - FBorderSpacing + 1;
        Y2 := 0;
        if FLineWidth > 1 then
          DrawOneRect(X1, Y1, X2, Y2 + FLineWidth - 1)
        else
          DrawOneLine(X1, Y1, X2, Y2);

      end else if FOrientation = stBottom then begin
        X1 := FBorderSpacing;
        Y1 := height - 1;
        X2 := width - FBorderSpacing + 1;
        Y2 := height - 1;
        if FLineWidth > 1 then
          DrawOneRect(X1, Y2 - FLineWidth, X2, Y1)
        else
          DrawOneLine(X1, Y1, X2, Y2);

      end;
    end;
  end;

  procedure DrawTicksAndText;

    procedure DrawOneTick(x1, y1, x2, y2: integer);
    begin
      With FBmpBuffer.Canvas do begin
        pen.color := FTicksColor;
        brush.color := FTicksColor;
        moveTo(x1, y1);
        lineTo(x2, y2);
      end;
    end;

    procedure DrawOneLed(x1, y1, x2, y2: integer; UseActiveCl: boolean);
    begin
      With FBmpBuffer.Canvas do begin
        if UseActiveCl then begin
          brush.color := FTicksActiveColor;
          pen.color := FTicksActiveColor;
        end else begin
          brush.color := FTicksInactiveColor;
          pen.color := FTicksInactiveColor;
        end;
        rectangle(x1, y1, x2, y2);

        // 3d effect ?
        if FTicksWidth > 2 then begin
          pen.color := clGray;
          moveTo(x1, y2);
          lineTo(x1, y1);
          lineTo(x2 - 1, y1);
          pen.color := clWhite;
          lineTo(x2 - 1, y2);
          lineTo(x1, y2);
        end;
      end;
    end;

  var
    i, cnt, xTick1, yTick1, xTick2, yTick2, xText, yText: integer;
    BaseIntervalVal: real;
    BaseIntervalPxl: real;
    TickText: string;
    UseActiveColor: boolean;
  begin
    cnt := FTicksCount;
    BaseIntervalVal := (FMaxValue - FMinValue) / (FTicksCount - 1);
    With FBmpBuffer.Canvas do begin
      pen.color := FTicksColor;
      font.Color := FFontColor;
      font.Name := FFontName;
      font.Size := FFontSize;
      for i := 0 to cnt - 1 do begin

        if FOrientation = stLeft then begin
          UseActiveColor := position >= (FMaxValue - (BaseIntervalVal * i));
          BaseIntervalPxl := (height - (FBorderSpacing * 2)) / (FTicksCount - 1);
          if FTextStyle = tsMinMaxValues then
            TickText := r2s(FDecimals, FMaxValue - (BaseIntervalVal * i))
          else if (FTextStyle = tsMinMaxText) then begin
            if i = 0 then
              TickText := FTextMax
            else if i = cnt - 1 then
              TickText := FTextMin
            else
              TickText := '';
          end;
          xTick1 := FTicksOffset + FLineWidth;
          yTick1 := FBorderSpacing + trunc(BaseIntervalPxl * i);
          xTick2 := FTicksOffset + FLineWidth + FTicksLength;
          yTick2 := yTick1;
          xText := FTicksOffset + FLineWidth + FTicksLength + FTextTickSpacing;
          if i = 0 then
            yText := yTick1 - trunc(textHeight(TickText) / 2) + FTextTopOffset
          else if i = pred(cnt) then
            yText := yTick1 - trunc(textHeight(TickText) / 2) - FTextBottomOffset
          else
            yText := yTick1 - trunc(textHeight(TickText) / 2);
          if FTicksWidth > 1 then
            DrawOneLed(xTick1, yTick1 - trunc(FTicksWidth / 2), xTick2, yTick2 + trunc(FTicksWidth / 2), UseActiveColor)
          else
            DrawOneTick(xTick1, yTick1, xTick2, yTick2);

        end else if FOrientation = stRight then begin
          UseActiveColor := position >= (FMaxValue - (BaseIntervalVal * i));
          BaseIntervalPxl := (height - (FBorderSpacing * 2)) / (FTicksCount - 1);
          if FTextStyle = tsMinMaxValues then
            TickText := r2s(FDecimals, FMaxValue - (BaseIntervalVal * i))
          else if (FTextStyle = tsMinMaxText) then begin
            if i = 0 then
              TickText := FTextMax
            else if i = pred(cnt) then
              TickText := FTextMin
            else
              TickText := '';
          end;
          xTick1 := width - FLineWidth - FTicksOffset - 1;
          yTick1 := FBorderSpacing + trunc(BaseIntervalPxl * i);
          xTick2 := width - FLineWidth - FTicksLength - FTicksOffset - 1;
          yTick2 := yTick1;
          xText := width - FLineWidth - FTicksOffset - FTicksLength - FTextTickSpacing - textWidth(TickText);
          if i = 0 then
            yText := yTick1 - trunc(textHeight(TickText) / 2) + FTextTopOffset
          else if i = pred(cnt) then
            yText := yTick1 - trunc(textHeight(TickText) / 2) - FTextBottomOffset
          else
            yText := yTick1 - trunc(textHeight(TickText) / 2);
          if FTicksWidth > 1 then
            DrawOneLed(xTick2, yTick1 - trunc(FTicksWidth / 2), xTick1, yTick2 + trunc(FTicksWidth / 2), UseActiveColor)
          else
            DrawOneTick(xTick1, yTick1, xTick2, yTick2);

        end else if FOrientation = stTop then begin
          UseActiveColor := position >= (BaseIntervalVal * i);
          BaseIntervalPxl := (width - (FBorderSpacing * 2)) / (FTicksCount - 1);
          if FTextStyle = tsMinMaxValues then
            TickText := r2s(FDecimals, (BaseIntervalVal * i) + FMinValue)
          else if (FTextStyle = tsMinMaxText) then begin
            if i = 0 then
              TickText := FTextMin
            else if i = cnt - 1 then
              TickText := FTextMax
            else
              TickText := '';
          end;
          xTick1 := FBorderSpacing + trunc(BaseIntervalPxl * i);
          yTick1 := FLineWidth + FTicksOffset;
          xTick2 := xTick1;
          yTick2 := FLineWidth + FTicksOffset + FTicksLength;
          yText := FLineWidth + FTicksLength + FTextTickSpacing + FTicksOffset;
          if i = 0 then
            xText := xTick1 - trunc(textWidth(TickText) / 2) + FTextLeftOffset
          else if i = pred(cnt) then
            xText := xTick1 - trunc(textWidth(TickText) / 2) - FTextRightOffset
          else
            xText := xTick1 - trunc(textWidth(TickText) / 2);
          if FTicksWidth > 1 then
            DrawOneLed(xTick1 - trunc(FTicksWidth / 2) - 1, yTick1, xTick2 + trunc(FTicksWidth / 2), yTick2, UseActiveColor)
          else
            DrawOneTick(xTick1, yTick1, xTick2, yTick2);

        end else if FOrientation = stBottom then begin
          UseActiveColor := position >= (BaseIntervalVal * i);
          BaseIntervalPxl := (width - (FBorderSpacing * 2)) / (FTicksCount - 1);
          if FTextStyle = tsMinMaxValues then
            TickText := r2s(FDecimals, (BaseIntervalVal * i) + FMinValue)
          else if (FTextStyle = tsMinMaxText) then begin
            if i = 0 then
              TickText := FTextMin
            else if i = cnt - 1 then
              TickText := FTextMax
            else
              TickText := '';
          end;
          xTick1 := FBorderSpacing + trunc(BaseIntervalPxl * i);
          yTick1 := height - FLineWidth - FTicksOffset - 1;
          xTick2 := xTick1;
          yTick2 := height - FLineWidth - FTicksOffset - FTicksLength;
          yText := height - FLineWidth - FTicksOffset - FTicksLength - FTextTickSpacing - textHeight(TickText);
          if i = 0 then
            xText := xTick1 - trunc(textWidth(TickText) / 2) + FTextLeftOffset
          else if i = pred(cnt) then
            xText := xTick1 - trunc(textWidth(TickText) / 2) - FTextRightOffset
          else
            xText := xTick1 - trunc(textWidth(TickText) / 2);
          if FTicksWidth > 1 then
            DrawOneLed(xTick2 - trunc(FTicksWidth / 2) - 1, yTick2, xTick1 + trunc(FTicksWidth / 2), yTick1 - 1, UseActiveColor)
          else
            DrawOneTick(xTick1, yTick1, xTick2, yTick2);
        end;


        if FShowText then begin
          Brush.color := FBGColor;
          TextOut(xText, yText, TickText);
        end;

      end;
    end;
  end;

  procedure DrawMarker;
  var
    xTick1, yTick1, xTick2, yTick2: integer;
    ratio: real;
  begin
    With FBmpBuffer.Canvas do begin
      try
        if (FPosition > FMinValue) and (FPosition < FMaxValue) then
          ratio := trunc((100 / ((FMaxValue - FMinValue) / (FPosition - FMinValue))));
      except
      end;

      if FOrientation = stLeft then begin

        // draw tick marker
        Pen.Color := FMarkerColor;
        xTick1 := FTicksOffset + FLineWidth;
        if (FPosition <= FMinValue) then
          yTick1 := height - FBorderSpacing - 1
        else if (FPosition >= FMaxValue) then
          yTick1 := FBorderSpacing
        else if (FPosition > FMinValue) and (FPosition < FMaxValue) then
          yTick1 := height - trunc((height - (FBorderSpacing * 2)) / 100 * ratio) - FBorderSpacing;
        xTick2 := FTicksOffset + FLineWidth + FTicksLength;
        yTick2 := yTick1;
        if FShowTickMarker then begin
          moveTo(xTick1, yTick1);
          lineTo(xTick2, yTick2);
        end;

        // draw in base line if line width > 2
        if FLineWidth > 2 then begin
          Pen.Color := FLineActiveColor;
          Brush.Color := FLineActiveColor;
          if yTick2 <= FBorderSpacing then
            yTick2 := FBorderSpacing + 1;
          rectangle(1, height - FBorderSpacing, FLineWidth - 1, yTick2);
        end;

      end else if FOrientation = stRight then begin

        // draw tick marker
        Pen.Color := FMarkerColor;
        xTick1 := width - FLineWidth - FTicksOffset;
        if (FPosition <= FMinValue) then
          yTick1 := height - FBorderSpacing - 1
        else if (FPosition >= FMaxValue) then
          yTick1 := FBorderSpacing
        else if (FPosition > FMinValue) and (FPosition < FMaxValue) then
          yTick1 := height - trunc((height - (FBorderSpacing * 2)) / 100 * ratio) - FBorderSpacing;
        xTick2 := width - FLineWidth - FTicksLength - FTicksOffset;
        yTick2 := yTick1;
        if FShowTickMarker then begin
          moveTo(xTick1, yTick1);
          lineTo(xTick2, yTick2);
        end;

        // draw in base line if line width > 2
        if FLineWidth > 2 then begin
          Pen.Color := FLineActiveColor;
          Brush.Color := FLineActiveColor;
          if yTick2 <= FBorderSpacing then
            yTick2 := FBorderSpacing + 1;
          rectangle(width - 1, height - FBorderSpacing, width - FLineWidth + 1, yTick2);
        end;

      end else if FOrientation = stTop then begin

        // draw tick marker
        Pen.Color := FMarkerColor;
        if (FPosition <= FMinValue) then
          xTick1 := FBorderSpacing
        else if (FPosition >= FMaxValue) then
          xTick1 := width - FBorderSpacing - 1
        else if (FPosition > FMinValue) and (FPosition < FMaxValue) then
          xTick1 := trunc((width - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing;
        yTick1 := FLineWidth + FTicksOffset;
        xTick2 := xTick1;
        yTick2 := FLineWidth + FTicksLength + FTicksOffset;
        if FShowTickMarker then begin
          moveTo(xTick1, yTick1);
          lineTo(xTick2, yTick2);
        end;

        // draw in base line if line width > 2
        if FLineWidth > 2 then begin
          Pen.Color := FLineActiveColor;
          Brush.Color := FLineActiveColor;
          if xTick2 <= FBorderSpacing then
            xTick2 := FBorderSpacing + 1;
          rectangle(FBorderSpacing + 1, 1, xTick2 + 1, FLineWidth - 1);
        end;

      end else if FOrientation = stBottom then begin

        // draw tick marker
        Pen.Color := FMarkerColor;
        if (FPosition <= FMinValue) then
          xTick1 := FBorderSpacing
        else if (FPosition >= FMaxValue) then
          xTick1 := width - FBorderSpacing - 1
        else if (FPosition > FMinValue) and (FPosition < FMaxValue) then
          xTick1 := trunc((width - (FBorderSpacing * 2)) / 100 * ratio) + FBorderSpacing;
        yTick1 := height - FLineWidth - FTicksOffset;
        xTick2 := xTick1;
        yTick2 := height - FLineWidth - FTicksLength - FTicksOffset;
        if FShowTickMarker then begin
          moveTo(xTick1, yTick1);
          lineTo(xTick2, yTick2);
        end;

        // draw in base line if line width > 2
        if FLineWidth > 2 then begin
          Pen.Color := FLineActiveColor;
          Brush.Color := FLineActiveColor;
          if xTick2 <= FBorderSpacing then
            xTick2 := FBorderSpacing + 1;
          rectangle(FBorderSpacing + 1, height - 2, xTick2 + 1, height - FLineWidth );
        end;

      end;

    end;
  end;

begin
  // background
  DrawBackGround;

  // draw base line
  DrawBaseLine;

  // draw ticks
  DrawTicksAndText;

  // marker
  DrawMarker;
end;

procedure TrmScale.SetFontColor(val: TColor);
begin
  if val <> FFontColor then begin
    FFontColor := val;
    paint;
  end;
end;

procedure TrmScale.SetFontName(val: string);
begin
  if val <> FFontName then begin
    FFontName := val;
    paint;
  end;
end;

procedure TrmScale.SetFontSize(val: word);
begin
  if val <> FFontSize then begin
    FFontSize := val;
    paint;
  end;
end;

procedure TrmScale.SetTextTickSpacing(val: word);
begin
  if val <> FTextTickSpacing then begin
    FTextTickSpacing := val;
    paint;
  end;
end;

procedure TrmScale.SetBorderSpacing(val: word);
begin
  if val <> FBorderSpacing then begin
    FBorderSpacing := val;
    paint;
  end;
end;

procedure TrmScale.SetTextLeftOffset(val: integer);
begin
  if val <> FTextLeftOffset then begin
    FTextLeftOffset := val;
    paint;
  end;
end;

procedure TrmScale.SetTextRightOffset(val: integer);
begin
  if val <> FTextRightOffset then begin
    FTextRightOffset := val;
    paint;
  end;
end;

procedure TrmScale.SetTextBottomOffset(val: integer);
begin
  if val <> FTextBottomOffset then begin
    FTextBottomOffset := val;
    paint;
  end;
end;

procedure TrmScale.SetTextTopOffset(val: integer);
begin
  if val <> FTextTopOffset then begin
    FTextTopOffset := val;
    paint;
  end;
end;

procedure TrmScale.SetMaxValue(val: single);
begin
  if val <> FMaxValue then begin
    FMaxValue := val;
    paint;
  end;
end;

procedure TrmScale.SetMinValue(val: single);
begin
  if val <> FMinValue then begin
    FMinValue := val;
    paint;
  end;
end;

procedure TrmScale.SetDecimals(Val: word);
begin
  if val <> FDecimals then begin
    FDecimals := val;
    paint;
  end;
end;

procedure TrmScale.SetPosition(val: single);
begin
  if val <> FPosition then begin
    FPosition := val;
    paint;
  end;
end;

procedure TrmScale.SetShowTickMarker(val: boolean);
begin
  if val <> FShowTickMarker then begin
    FShowTickMarker := val;
    paint;
  end;
end;

procedure TrmScale.SetMarkerColor(val: TColor);
begin
  if val <> FMarkerColor then begin
    FMarkerColor := val;
    paint;
  end;
end;

end.
